﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Navigation;
using Syncfusion.UI.Xaml.Kanban;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace KanbanSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

    }

    public class TaskDetails
    {
        public TaskDetails()
        {
            Tasks = new ObservableCollection<CardItem>();

            CardItem task = new CardItem();
            task.Title = "UWP Issue";
            task.ID = "6593";
            task.Description = "Sorting is not working properly in DateTimeAxis";
            task.Category = "Postponed";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "UWP Issue";
            task.ID = "651";
            task.Description = "Crosshair label template not visible in UWP.";
            task.Category = "Open";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "Kanban Feature";
            task.ID = "25678";
            task.Description = "Need to prepare SampleBrowser sample";
            task.Category = "InProgress";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "WP Issue";
            task.ID = "1254";
            task.Description = "HorizontalAlignment for tooltip is not working";
            task.Category = "InProgress";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "WPF Issue";
            task.ID = "28066";
            task.Description = "In minimized state, first and last segment have incorrect spacing";
            task.Category = "Review";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "WPF Issue";
            task.ID = "29477";
            task.Description = "Null reference exception thrown in line chart";
            task.Category = "Review";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "WPF Issue";
            task.ID = "29574";
            task.Description = "Minimum and maximum property are not working in dynamic update";
            task.Category = "Review";
            Tasks.Add(task);

            task.Title = "Kanban Feature";
            task.ID = "27654";
            task.Description = "Need to prepare demo sample";
            task.Category = "Done";
            task.ColorKey = "Low";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "Kanban Feature";
            task.ID = "29574";
            task.Description = "Need to implement tooltip support for SfKanban";
            task.Category = "Review";
            Tasks.Add(task);

            task = new CardItem();
            task.Title = "New Feature";
            task.ID = "29574";
            task.Description = "Dragging events support for SfKanban";
            task.Category = "Closed";
            Tasks.Add(task);

            Tasks.Add(task);
        }
        public ObservableCollection<CardItem> Tasks { get; set; }

    }

    public class CardItem : INotifyPropertyChanged
    {
        public const string ProgressCategoryRequired = "On Progress";

        private string _category;
        private string _colorKey;
        private string _description;
        private string _iD;
        private string _tags;
        private string _title;
        private double _progress;

        // Summary:
        //     Gets or sets the object which indicates card's section.
        public string Category
        {
            get { return _category; }
            set
            {
                bool changed = !Equals(_category, value);
                if (changed)
                {
                    _category = value;
                    this.OnPropertyChanged(nameof(Category));
                }
            }
        }

        //
        // Summary:
        //     Gets or sets the object that used to set indication color for card.
        public string ColorKey
        {
            get { return _colorKey; }
            set
            {
                bool changed = !Equals(_colorKey, value);
                if (changed)
                {
                    _colorKey = value;
                    this.OnPropertyChanged(nameof(ColorKey));
                }
            }
        }

        //
        // Summary:
        //     Gets or sets the string value which is used to display card's content.
        public string Description
        {
            get { return _description; }
            set
            {
                bool changed = !Equals(_description, value);
                if (changed)
                {
                    _description = value;
                    this.OnPropertyChanged(nameof(Description));
                }
            }
        }

        public double Progress
        {
            get
            {
                return _progress;
            }
            set
            {
                bool changed = _progress != value;

                if (changed)
                {
                    _progress = value;
                    this.OnPropertyChanged(nameof(Progress));
                }
            }
        }

        //
        // Summary:
        //     Gets or sets the string value which indicates identity number of the card.
        public string ID
        {
            get { return _iD; }
            set
            {
                bool changed = !Equals(_iD, value);
                if (changed)
                {
                    _iD = value;
                    this.OnPropertyChanged(nameof(ID));
                }
            }
        }

        //
        // Summary:
        //     Gets or sets the string array which indicates tag value of the card.
        public string Tags
        {
            get { return _tags; }
            set
            {
                bool changed = !Equals(_tags, value);
                if (changed)
                {
                    _tags = value;
                    this.OnPropertyChanged(nameof(Tags));
                }
            }
        }
        //
        // Summary:
        //     Gets or sets the string value which is used to display card's header string.
        public string Title
        {
            get { return _title; }
            set
            {
                bool changed = !Equals(_title, value);
                if (changed)
                {
                    _title = value;
                    this.OnPropertyChanged(nameof(Title));
                }
            }
        }

        public object Tag { get; set; }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }


    public class KanbanColumnAdv : KanbanColumn
    {

        #region Fields

        private ControlTemplate collapsedTemplate;
        private ControlTemplate expandedTemplate;

        #endregion

        #region ctor
        public KanbanColumnAdv()
        {
            PointerReleased += KanbanColumnAdv_PointerReleased;
            collapsedTemplate = KanbanDictionaries.GenericCommonDictionary["CollapsedTemplate"] as ControlTemplate;
            expandedTemplate = KanbanDictionaries.GenericCommonDictionary["ExpandedTemplate"] as ControlTemplate;
        }

        #endregion


        #region Properties

        public new bool IsExpanded
        {
            get { return (bool)GetValue(IsExpandedProperty); }
            set { SetValue(IsExpandedProperty, value); }
        }

        public new static readonly DependencyProperty IsExpandedProperty = DependencyProperty.Register(
            "IsExpanded", typeof(bool), typeof(KanbanColumnAdv), new PropertyMetadata(true, OnIsExpandedChanged));


        #endregion


        #region Methods

        private static void OnIsExpandedChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            KanbanColumnAdv column = d as KanbanColumnAdv;

            column.ExpandedChanged((bool)e.NewValue);
        }

        private void ExpandedChanged(bool isExpanded)
        {
            FieldInfo sbInfo = typeof(KanbanColumn).GetField("storyBoard", BindingFlags.Instance |
                                                                              BindingFlags.NonPublic |
                                                                              BindingFlags.Public);
            if (sbInfo != null)
            {
                Storyboard storyBoard = sbInfo.GetValue(this) as Storyboard;
                storyBoard.Stop();
            }

            if (Tags != null)
                Tags.IsExpanded = IsExpanded;

            if (!isExpanded)
            {
                ClearValue(Control.WidthProperty);
                Width = 50;

                ClearValue(Control.TemplateProperty);

                if (collapsedTemplate != null)
                    Template = collapsedTemplate;

                foreach (KanbanCardItem card in Cards)
                {
                    card.Visibility = Visibility.Collapsed;
                }
            }
            else
            {
                ClearValue(Control.WidthProperty);

                PropertyInfo propInfo = typeof(KanbanColumn).GetProperty("Area", BindingFlags.Instance |
                                                                                 BindingFlags.NonPublic |
                                                                                 BindingFlags.Public);
                
                SfKanban area = propInfo.GetValue(this) as SfKanban;
                
                if (area != null)
                {
                    Binding binding = new Binding()
                    {
                        Source = area,
                        Path = new PropertyPath("ActualColumnWidth")
                    };
                    SetBinding(Control.WidthProperty, binding);
                }

                if (expandedTemplate != null)
                    Template = expandedTemplate;
                foreach (KanbanCardItem card in Cards)
                {
                    card.Visibility = Visibility.Visible;
                }
            }
        }

        private void KanbanColumnAdv_PointerReleased(object sender, PointerRoutedEventArgs e)
        {
            if (e.OriginalSource is Border &&
                    ((e.OriginalSource as Border).Name == "CollapsedIcon"))
            {
                IsExpanded = false;
            }
            else if (!IsExpanded)
            {
                IsExpanded = true;
            }
        }

        #endregion
    }

}
